import logging
from datetime import datetime, date
import os
import torch
import time



def setup_logging(logger_name, path, level):
    logger = logging.getLogger(logger_name)
    logger.setLevel(level)
    print(f"current working directory is {os.getcwd()}")
    print(f"path of log file {path}")
    fh = logging.FileHandler(path)
    fh.setLevel(level)
    formatter = logging.Formatter('%(message)s')
    fh.setFormatter(formatter)
    logger.addHandler(fh)
    now=datetime.now()
    logger.info(now.strftime("%Y-%m-%d-%H:%M")+"\n")
    return logger

def log_message(logger, message, level=logging.WARNING):
    #logger = logging.getLogger(logger_name)
    now=datetime.now()
    if level == logging.INFO:
        logger.info(now.strftime("%H:%M")+"\t" + message)
    else:
        logger.warning(now.strftime("%H:%M")+"\t" + message)

def format_metrics(metrics):
    formatted_str = ""
    for metric in metrics:
        formatted_str = formatted_str + f"{metric}: {metrics[metric]:2.2f}\t"
    return formatted_str

def format_params (params):
    formatted_str = ""
    for param_key in params:
        formatted_str = formatted_str + f"{param_key}: {params[param_key]}\t"
    return formatted_str

def log_metrics(logger, metrics, level = logging.WARNING):

    message = format_metrics(metrics)
    log_message(logger, message, level)

def log_params(logger, params, level = logging.WARNING):

    params_message = format_params (params)
    log_message(logger, params_message, level= logging.WARNING)

def log_memory_usage(logger, state):
    total_free_gpu_memory, total_gpu_memory = torch.cuda.mem_get_info()
    log_message(logger, state, level=logging.INFO)
    log_message(logger, f"Total free GPU memory: {round(total_free_gpu_memory * 1e-9, 3)} GB", level=logging.INFO)
    log_message(logger,f"Total GPU memory: {round(total_gpu_memory * 1e-9, 3)} GB", level=logging.INFO)

def measure_time():
    return time.time()

def log_time(logger, start_time, message):
    elapsed_time = time.time() - start_time
    log_message(logger, f"meesage took {elapsed_time}")


import logging

import numpy as np
import random
import torch


from argparse import *

from random import randint
from collections import Counter
from sklearn.metrics import f1_score, accuracy_score
from torch import nn
import torch

from torch.utils.data import DataLoader
from transformers import  AutoModelForSequenceClassification, AutoTokenizer, AdamW
from transformers import set_seed

from few_shot_priming.config import *
from few_shot_priming.mylogging import *
from few_shot_priming.experiments import *




class Dataset(torch.utils.data.Dataset):
    def __init__(self,dataset, path=None, model='bert-base-cased'):
        topics, texts, stances = list(dataset["topic"].values), list(dataset["text"].values), list(dataset["stance"].values)
        print(topics)
        if path:
            tokenizer = AutoTokenizer.from_pretrained(path)
        else:
            tokenizer = AutoTokenizer.from_pretrained(model)
        self.labels = stances
        self.texts = tokenizer(topics, texts, padding=True
                               , truncation=True, return_tensors="pt")
        print(f"size of labels {len(self.labels)} and size of texts is {len(self.texts)}")

    def classes(self):
        return self.labels

    def __len__(self):
        return len(self.labels)

    def get_batch_labels(self, idx):
        return np.array(self.labels[idx])

    def get_batch_texts(self, idx):
        return {"input_ids":self.texts["input_ids"][idx],"attention_mask": self.texts["attention_mask"][idx]}

    def __getitem__(self, idx):
        batch_texts = self.get_batch_texts(idx)
        batch_y = self.get_batch_labels(idx)
        return batch_texts, batch_y


class DeBERTaClassifier(nn.Module):
    def __init__(self, hyperparameters, path=None, num_labels=2):
        super(DeBERTaClassifier, self).__init__()
        self.lr = hyperparameters["learning-rate"]
        self.batch_size = hyperparameters["batch-size"]
        if path:
            self.deberta = AutoModelForSequenceClassification.from_pretrained(path, num_labels=num_labels, ignore_mismatched_sizes=True)
        else:
            self.deberta = AutoModelForSequenceClassification.from_pretrained('microsoft/deberta-base', num_labels=num_labels, ignore_mismatched_sizes=True)
        self.labels = {"Con": 0, "Pro":1}

    def forward(self, input_ids, mask):
        pooled_output = self.deberta(input_ids=input_ids,  return_dict=False)

        return pooled_output

class BertClassifier(nn.Module):
    def __init__(self, hyperparameters, path=None, num_labels=2):
        super(BertClassifier, self).__init__()
        self.lr = hyperparameters["learning-rate"]
        self.batch_size = hyperparameters["batch-size"]
        if path:
            self.bert = AutoModelForSequenceClassification.from_pretrained(path, num_labels=num_labels, ignore_mismatched_sizes=True)
        else:
            self.bert = AutoModelForSequenceClassification.from_pretrained('bert-base-cased', num_labels=num_labels, ignore_mismatched_sizes=True)
        self.labels = {"Con": 0, "Pro":1}


    def forward(self, input_ids, mask):
        pooled_output = self.bert(input_ids=input_ids, attention_mask=mask, return_dict=False)
        return pooled_output




def parse_args():
    """
    Parse the arguments of the scripts
    :return:
    """
    parser = ArgumentParser()
    parser.add_argument("--validate", action="store_true")
    parser.add_argument("--test", action="store_true")
    parser.add_argument("--offline", action="store_true")
    parser.add_argument("--random", action="store_true")
    parser.add_argument("--majority", action="store_true")
    parser.add_argument("--ibm-api", action="store_true")
    parser.add_argument("--vast", action="store_true")
    parser.add_argument("--perspectrum", action="store_true")

    return parser.parse_args()

def run_finetuning_experiment_baseline(config=None, params=None,  experiment="ibmsc", validate=False, offline=True,
                                       logger=None, debug=False):
    splits = load_splits(experiment, oversample=True, validate=validate, debug=debug)
    use_cuda = torch.cuda.is_available()

    if use_cuda:
        log_message(logger, "-.-.-.- using cuda -.-.-.-.-", level=logging.INFO)
    else:
        log_message(logger, "-.-..-. using cpu -.-.-.-.-..", level=logging.INFO)

    path = None
    if offline:
        #save_pre_trained_model()
        path = config["model-path"]
    model_name = config["model-name"]
    path_model_fine_tuned = config["model-path-fine-tuned"]

    if not params:
        params = get_baseline_best_params(experiment)
    else:
        log_message(logger, "Reading params from config file", level=logging.INFO)

    train_dataset = Dataset(splits["training"], path=path, model=model_name)
    if validate:
        experiment_type = "validate"
        test_dataset = Dataset(splits["validation"], path=path, model=model_name)
    else:
        experiment_type = "test"
        test_dataset = Dataset(splits["test"], path=path, model=model_name)

    train_dataloader = DataLoader(train_dataset, batch_size=params["batch-size"], shuffle=True)
    test_dataloader = DataLoader(test_dataset, batch_size=params["batch-size"], shuffle=True)
    eval_dataloader = DataLoader(test_dataset, batch_size=1, shuffle=False)

    device = torch.device("cuda" if use_cuda else "cpu")
    if experiment == "ibmsc" or experiment == "perspectrum":
        num_labels = 2
    else:
        num_labels = 3
    if model_name.startswith("bert"):
        model = BertClassifier(params, path=path, num_labels=num_labels)
    else:
        model = DeBERTaClassifier(params, path=path, num_labels=num_labels)

    if use_cuda:
        model = model.cuda()
    optimizer = AdamW(model.parameters(), lr=float(params["learning-rate"]))
    epochs = params["epochs"]
    criterion = nn.CrossEntropyLoss().cuda()

    best_f1 = -1
    best_epoch = 0
    best_metrics = None

    model.train()
    metrics = {}

    for epoch in range(epochs):
        train_loss = 0
        all_test_labels = []
        all_test_preds = []
        model.train()
        for step, (train_input, train_label) in enumerate(train_dataloader):
            train_label = train_label.to(device)
            mask = train_input["attention_mask"].to(device)
            input_ids = train_input["input_ids"].to(device)
            output = model(input_ids, mask)
            batch_loss = criterion(output[0], train_label)
            #log_message(logger, str(output[0].shape), level=logging.INFO)
            #log_message(logger, str(train_label.shape) ,level=logging.INFO)
            train_loss += batch_loss.item()
            batch_loss.backward()
            optimizer.step()
            optimizer.zero_grad()
            metrics["train/loss"] = train_loss / (step +1)

        test_loss = 0
        model.eval()
        for step, (test_input, test_labels) in enumerate(test_dataloader):
            test_labels = test_labels.to(device)
            mask = test_input["attention_mask"].to(device)
            input_ids = test_input["input_ids"].to(device)
            output = model(input_ids, mask)
            batch_loss = criterion(output[0], test_labels)
            test_loss += batch_loss.item()
            metrics[f"{experiment_type}/loss"] = test_loss / (step+1)

        for step, (test_input, test_labels) in enumerate(eval_dataloader):
            test_labels = test_labels.to(device)
            mask = test_input["attention_mask"].to(device)
            input_ids = test_input["input_ids"].to(device)
            output = model(input_ids, mask)
            predictions = torch.argmax(output[0],dim=1)
            all_test_preds.extend(predictions.cpu())
            all_test_labels.extend(test_labels.cpu().tolist())

        test_accuracy = accuracy_score(all_test_labels, all_test_preds)
        if experiment == "vast":
            f1s = f1_score(all_test_labels, all_test_preds,average=None, labels=[0, 1, 2])
            neutral_f1 = f1s[2]
            metrics[f"{experiment_type}/neutral-f1"] = neutral_f1
        else:
            f1s = f1_score(all_test_labels, all_test_preds,average=None, labels=[0, 1])

        con_f1 = f1s[0]
        pro_f1 = f1s[1]
        macro_f1 = f1_score(all_test_labels, all_test_preds, average="macro")
        metrics[f"{experiment_type}/macro-f1"] = macro_f1
        metrics[f"{experiment_type}/pro-f1"] = pro_f1
        metrics[f"{experiment_type}/con-f1"] = con_f1
        metrics[f"{experiment_type}/accuracy"] = test_accuracy

        if macro_f1 > best_f1:
            best_f1 = macro_f1
            best_metrics = metrics
            best_epoch = epoch
            torch.save({"model_state_dict":model.state_dict()}, path_model_fine_tuned)

    if best_epoch and best_metrics:
        log_message(logger, f"best epoch is {best_epoch}", level=logging.WARNING)
        log_metrics(logger, best_metrics, level=logging.WARNING)
    return metrics


def baseline(config, experiment, offline=True, validate=True, majority=False):
    splits = load_splits(experiment, validate=validate, oversample=False)
    if validate:
        test_split = splits["validation"]
    else:
        test_split = splits["test"]

    if offline:
        #save_pre_trained_model()
        path = config["model-path"]

    test_dataset = Dataset(test_split, path=path)
    test_dataloader = DataLoader(test_dataset, batch_size=8, shuffle=True)
    predictions = []
    labels = []
    for step, (test_input, test_labels) in enumerate(test_dataloader):
        test_labels = test_labels.numpy()
        labels.extend(test_labels)
    c = Counter(labels)
    labels = []
    majority_label = c.most_common()[0][0]
    for step, (test_input, test_labels) in enumerate(test_dataloader):
        test_labels = test_labels.numpy()
        labels.extend(test_labels)
        if majority:
            predictions.extend([majority_label for _ in test_labels])
        else:
            predictions.extend([randint(0,1) for _ in test_labels])

    print(f"size of test is {test_labels}")
    pro_f1 = f1_score(labels, predictions, average=None, labels= [1])
    con_f1 = f1_score(labels, predictions, average=None, labels =[0])
    if experiment == "vast":
        neutral_f1 = f1_score(labels, predictions, average=None, labels =[2])
        neutral_f1 = neutral_f1[0]
    else:
        neutral_f1 = None
    macro_f1 = f1_score(labels, predictions, average="macro")
    accuracy = accuracy_score(labels, predictions)

    return accuracy, pro_f1[0], con_f1[0], neutral_f1, macro_f1


if __name__ == "__main__":
    args = parse_args()
    config = get_baseline_config()
    if args.vast:
        experiment = "vast"
    elif args.perspectrum:
        experiment = "perspectrum"
    else:
        experiment = "ibmsc"

    if args.random:
        accuracy, pro_f1, con_f1, neutral_f1, macro_f1  = baseline(config, experiment= experiment, offline=args.offline, validate=args.validate)
        print(f"random baseline: accuracy {accuracy}, macro-f1 {macro_f1}, pro f1 {pro_f1}, con f1 {con_f1}, "
              f" neutral f1 {neutral_f1}")
    elif args.majority:
        accuracy, pro_f1, con_f1, neutral_f1, macro_f1  = baseline(config, experiment= experiment, offline=args.offline, validate=args.validate, majority=True)
        print(f"majority baseline: accuracy {accuracy}, macro-f1 {macro_f1}, pro f1 {pro_f1}, con f1 {con_f1} neutral f1 {neutral_f1}")
    elif args.ibm_api:
        accuracy, pro_f1, con_f1, neutral_f1, macro_f1  = run_ibm_baseline(config, experiment= experiment, validate=args.validate)
        print(f"ibm api: accuracy {accuracy}, macro-f1 {macro_f1}, pro f1 {pro_f1}, con f1 {con_f1} neutral f1 {neutral_f1}")




import pandas as pd
import numpy as np
from sentence_transformers import SentenceTransformer, util
from collections import defaultdict
from gensim.corpora import Dictionary
from gensim.models import LdaModel
from scipy.spatial.distance import cosine


def calc_similarity_sentence_transformer(df_test, df_training):
    model = SentenceTransformer('all-mpnet-base-v2')
    df_test["topic-text"] = df_test.apply(lambda record: record["topic"] + "[SEP]" + record["text"], axis=1)
    df_training["topic-text"] = df_training.apply(lambda record: record["topic"] + "[SEP]" + record["text"],  axis=1)
    test_text = df_test["topic-text"]
    training_text = df_training["topic-text"]
    #test_text = df_test["text"]
    #training_text = df_training["text"]
    test_embeddings = model.encode(test_text.values.tolist())
    training_embeddings = model.encode(training_text.values.tolist())
    cosine_scores = util.cos_sim(test_embeddings, training_embeddings)
    positve_cosine_scores = np.absolute(cosine_scores)
    similarities = defaultdict(dict)
    i=0
    j=0
    for _, test_record in df_test.iterrows():
        for _, train_record in df_training.iterrows():
            similarities[test_record["id"]][train_record["id"]] = float(positve_cosine_scores[i,j])
            j = j + 1
        i = i + 1
        j = 0
    return similarities





import os
import os
import random

import pandas as pd


from few_shot_priming.config import *
from transformers import set_seed


def init_reproducible(seed):
    set_seed(seed)

def get_model_instruction_fine_tuninig_from(args):
    if args.alpaca_7b:
        return "alpaca-7b"
    elif args.mistral_7b_instruct:
        return "mistral-7b-instruct"
    elif args.vicuna_7b:
        return "vicuna-7b"

def get_model_prompting_from(args):
    if args.llama2_7b:
        return "llama2-7b"
    elif args.opt_30b:
        return "opt-30b"
    elif args.gpt2:
        return "gpt2"
    elif args.alpaca_7b:
        return "alpaca-7b"
    elif args.mistral_7b_instruct:
        return "mistral-7b-instruct"
    elif args.vicuna_7b:
        return "vicuna-7b"
    elif args.gemma_7b_instruct:
        return "gemma-7b-instruct"

def get_experiment_from(args):

    if args.vast:
        experiment = "vast"
    elif args.perspectrum:
        experiment = "perspectrum"
    elif args.ibmsc:
        experiment = "ibmsc"

    return experiment

def get_sampling_strategy_from(args):
    if args.similar_examples:
        sampling_strategy = "similar"
    elif args.diverse_examples:
        sampling_strategy = "diverse-examples"
    else:
        sampling_strategy = None
    return sampling_strategy

def get_similarity_from(args):
    if args.parse_tree_kernel:
        similarity = "parse-tree-kernel"
    elif args.ctm:
        similarity = "ctm"
    elif args.sentence_transformer:
        similarity = "sentence-transformer"
    else:
        similarity = None
    return similarity

def get_experiment_type_from(args):
    if args.validate:
        experiment_type = "validation"
    else:
        experiment_type= "test"
    return experiment_type

def get_sampling_strateg_property_from(args):
    if args.path_similar_examples:
        return args.path_similar_examples.split("/")[-1].replace(".csv","").replace(".tsv","")

def get_model_name(config):
    model_name = config["model-name"]
    return model_name

def get_run_name(args, config, prompting_type):

    experiment = get_experiment_from(args)
    experiment_type = get_experiment_type_from(args)
    sampling_strategy = get_sampling_strategy_from(args)
    sampling_strategy_property = get_sampling_strateg_property_from(args)
    model_name = get_model_name(config)
    if "/" in model_name:
        model_name = model_name.split("/")[-1]
    if prompting_type =="prompt-fine-tuning" and args.no_fine_tuning:
            model_name = model_name + "no-fine-tuning"
    if prompting_type == "prompt" and args.openai:
        model_name = "openai"
    few_shot_size = get_few_shot_size(experiment)
    if prompting_type =="prompt-fine-tuning" and args.optimize:
        run = f"optimize-hyperparameters-few-shot-k-{few_shot_size}"
    elif args.analyze_k:
        run = "analyze-k"
    elif args.analyze_prime_similarity:
        run =f"analyze-prime-similarity-{few_shot_size}"
    elif args.analyze_topic_similarity:
        run =f"analyze-topic-similarity-{few_shot_size}"
    elif args.analyze_topic_count:
        run =f"analyze-topic-count-k-{few_shot_size}"
    elif prompting_type =="prompt-fine-tuning" and args.all_training_data:
        run = f"all-training-data"
    else:
        run = f"k-{few_shot_size}"

    if sampling_strategy:
        if sampling_strategy_property:
            if run:
                results_name = f"{prompting_type}-{experiment}-{experiment_type}-{model_name}-{run}-{sampling_strategy_property}"
            else:
                results_name = f"{prompting_type}-{experiment}-{experiment_type}-{model_name}-{sampling_strategy_property}"
        elif run:
            results_name = f"{prompting_type}-{experiment}-{experiment_type}-{model_name}-{run}-{sampling_strategy}"
        else:
            results_name = f"{prompting_type}-{experiment}-{experiment_type}-{model_name}-{sampling_strategy}"


    else:
        if run:
            results_name = f"{prompting_type}-{experiment}-{experiment_type}-{model_name}-{run}-random-examples"
        else:
            results_name = f"{prompting_type}-{experiment}-{experiment_type}-{model_name}-random-examples"
    return results_name




def average_seeds(df_results):
    #metric_columns = [column for column in df_results.columns if column.startswith("validation") or column.startswith("test")]
    if "seed" in df_results.columns:
        if "k" in df_results.columns:
            df_average = df_results.groupby("k").mean().reset_index()
            df_average["seed"]= "average"
            df_results = pd.concat([df_results, df_average])
        elif "topic-count" in df_results.columns:
            df_average = df_results.groupby("topic-count").mean().reset_index()
            df_average["seed"]= "average"
            df_results = pd.concat([df_results, df_average])
        elif "max-similarity" in df_results.columns:
            df_average = df_results.groupby("max-similarity").mean().reset_index()
            df_average["seed"]= "average"
            df_results = pd.concat([df_results, df_average])
        else:
            df_average = df_results.mean()
            df_average["seed"]="average"
            df_results.loc["average"]=df_average



    return df_results

def get_max_topic_count(experiment, is_validate):
    """
    get the maximum count of unique topics in a dataset
    :param experiment:
    :param is_validate:
    :return:
    """
    path_source, path_training, path_validation, path_test = get_experiment_paths(experiment)
    df_training = pd.read_csv(path_training, sep=",")
    df_validation = pd.read_csv(path_validation, sep=",")
    if not is_validate:
        df_training = pd.concat([df_training, df_validation])
    if experiment == "ibmsc":
        topic_label = "topicText"
    else:
        topic_label = "topic_str"
    return df_training[topic_label].nunique()

def get_topic_count_range(experiment, is_validate):
    max_topic_count = get_max_topic_count(experiment, is_validate)
    if experiment == "ibmsc":
        topic_counts = list(range(1, max_topic_count+1, 5))
        topic_counts.append(max_topic_count)
    else:
        if is_validate:
            topic_counts = [5, 50, 100, 200, 300,  max_topic_count]
        else:
            topic_counts = [5, 1000, 2000, 3000, 4000, max_topic_count]
    return topic_counts

def decide_vast_shot_size(few_shot_size):
    if few_shot_size == 2:
        class_few_shot_size = 0
    else:
        class_few_shot_size = few_shot_size // 3
    random_value = random.randint(0,2)
    neutral_shot_size = class_few_shot_size
    pro_shot_size = class_few_shot_size
    con_shot_size = class_few_shot_size


    if few_shot_size % 3 == 2:
        if random_value == 0:
            pro_shot_size = pro_shot_size + 1
            neutral_shot_size = neutral_shot_size+ 1

        elif random_value == 1:
            con_shot_size = con_shot_size + 1
            neutral_shot_size = neutral_shot_size+ 1
        else:
            pro_shot_size = pro_shot_size+ 1
            con_shot_size = con_shot_size+ 1

    else:

        if random_value == 0:
            con_shot_size = con_shot_size + 1
        elif random_value == 1:
            pro_shot_size = pro_shot_size + 1
        else:
            neutral_shot_size = neutral_shot_size + 1

    assert (con_shot_size+pro_shot_size + neutral_shot_size == few_shot_size)

    return con_shot_size, pro_shot_size, neutral_shot_size


import unittest
from few_shot_priming.argument_sampling.stance_priming import *
class StancePrimingUnitTest(unittest.TestCase):
    def testPercentilesCalc(self):
        path_similarity = "/home/yamen/tmp/perspectrum-stance-similarties.json"
        thresholds = get_thresholds_for_percentiles(path_similarity, 10)
        print(thresholds)



import json
import logging
import math
import sys
import numpy as np

import pandas as pd
from sentence_transformers import *
from sklearn.neighbors import NearestCentroid
from sklearn.cluster import AgglomerativeClustering
from few_shot_priming.config import *
from few_shot_priming.experiments import *
from few_shot_priming.mylogging import *

from few_shot_priming.utils import *


def get_embeddings(df_training):
    model = SentenceTransformer('all-mpnet-base-v2')
    df_training["topic-text"] = df_training.apply(lambda record: record["topic"] + "[SEP]" + record["text"], axis=1)

    training_text = df_training["topic-text"]
    topics = df_training["topic"].values.tolist()
    training_embeddings = model.encode(training_text.values.tolist())
    ids = df_training["id"].values.tolist()
    return training_embeddings, ids, topics

def save_embeddings(embeddings, ids, topics, path):
    ids = np.reshape(ids, (len(ids),1))
    vectors_with_ids = np.hstack((ids, embeddings))
    np.savetxt(path+"/embeddings.txt",vectors_with_ids, delimiter=",")
    with open(path+"/topics.json", "w") as file:
        json.dump(topics, file)



def load_embeddings(path):
    vectors = np.genfromtxt(path+"/embeddings.txt", delimiter=",")
    with open(path+"/topics.json", "r") as file:
        topics = json.load(file)
    return vectors[:,1:], np.uint32(vectors[:,0]), topics

def cluster(vectors, ids, count_of_samples_per_cluster, **args):
    clustering = AgglomerativeClustering(**args).fit(vectors)
    labels = clustering.labels_

    clf = NearestCentroid()
    clf.fit(vectors, labels)
    samples = []
    for centriod in clf.centroids_:
        distances = []
        label = clf.predict([centriod])
        cluster_instances = [l==label[0] for l in labels]

        for i, vector in enumerate(vectors):
            if cluster_instances[i]:
                distances.append((i, np.linalg.norm(centriod-vector)))
        distances = sorted(distances, key= lambda element_id: element_id[1])
        if count_of_samples_per_cluster > len(distances):
            count_of_samples_per_cluster = len(distances)
        sampled_indices = [element_id[0] for element_id in distances[:count_of_samples_per_cluster]]
        samples.extend([(ids[sampled_index], labels[sampled_index]) for sampled_index in sampled_indices])

    return labels, samples


def find_diverse_examples(experiment, experiment_type, k, count_of_samples_per_cluster, drop_duplicate_perspectives, topic_count=None, logger=None):
    if experiment_type=="validation":
        validate=True
    else:
        validate=False

    log_message(logger, f"clustering on {experiment} {experiment_type} {k} few shots", logging.INFO)
    dataset = load_splits(experiment, oversample=False, validate=validate, drop_duplicate_perspectives=drop_duplicate_perspectives, topic_count=topic_count)
    df_training = dataset["training"]
    embeddings, ids, topics = get_embeddings(df_training)
    if k > len(df_training):
        k = len(df_training)

    labels, centroid_samples = cluster(embeddings, ids, count_of_samples_per_cluster, n_clusters=k)
    log_message(logger, f"found {len(set(labels))} clusters", logging.INFO)
    centroid_ids = [centroid_sample[0] for centroid_sample in centroid_samples]
    df_diverse= df_training[df_training["id"].isin(centroid_ids)]
    mapping =dict(centroid_samples)
    df_diverse["cluster"] = df_diverse["id"].apply(lambda x: mapping[x])
    return df_diverse


def save_diverse_examples(experiment, experiment_type, ks, count_of_samples_per_cluster,  drop_duplicate_perspectives=False, topic_counts = None, logger=None):
    if topic_counts:
        path = get_diverse_example_path(experiment, experiment_type, topic_counts)

    else:
        path = get_diverse_example_path(experiment, experiment_type, )

    if drop_duplicate_perspectives:
        path = path.replace(".tsv", "no-duplicate.tsv")
    log_message(logger, f"saving to path {path}")
    all_diverse_samples = []
    if topic_counts:
        log_message(logger, f"performing clustering on {list(topic_counts)}")
        assert (len(ks)==1)
        k = ks[0]

        for topic_count in topic_counts:
            log_message(logger, f"performing clustering on  {topic_count}", logging.INFO)
            df_diverse = find_diverse_examples(experiment, experiment_type, k,count_of_samples_per_cluster,drop_duplicate_perspectives, topic_count, logger=logger)
            df_diverse["k"] = k
            df_diverse["topic-count"] = topic_count
            all_diverse_samples.append(df_diverse)
    else:
        for k in ks:
            df_diverse = find_diverse_examples(experiment, experiment_type, k, count_of_samples_per_cluster, drop_duplicate_perspectives, topic_count=None, logger=logger)
            df_diverse["k"]=k
            all_diverse_samples.append(df_diverse)

    df_all_diverse_examples = pd.concat(all_diverse_samples)

    df_all_diverse_examples.to_csv(path, sep="\t")

def sample_diverse_examples(experiment, experiment_type, k, logger, topic_count=None, drop_duplicate_perspectives=False):
    path = get_diverse_example_path(experiment, experiment_type, topic_count, drop_duplicate_perspectives)
    df_diverse_examples = pd.read_csv(path, sep="\t")

    if topic_count:
        df_diverse_examples = df_diverse_examples[df_diverse_examples["topic-count"] == topic_count]

    if experiment == "ibmsc" or experiment =="perspectrum":
        label_ratio = {"validation": 0.5, "test": 0.5}


    log_message(logger,f"sampling pro, con, neutral arguments for {k} few shots", level= logging.INFO)
    if experiment == "ibmsc" or experiment =="perspectrum":
        pro_count = math.floor(k * label_ratio[experiment_type])
        con_count = k - pro_count
    else:

        #con_count, pro_count, neutral_count = decide_vast_shot_size(k)
        con_count, pro_count, neutral_count = k/3, k/3, k/3

    df_diverse_examples = df_diverse_examples[df_diverse_examples["k"] == k]
    instances = []
    for cluster, df_cluster in df_diverse_examples.groupby("cluster"):
        con_arguments = df_cluster[df_cluster["stance"] == 0]
        pro_arguments = df_cluster[df_cluster["stance"] == 1]
        if experiment == "vast":
            neutral_arguments = df_cluster[df_cluster["stance"] == 2]
        if experiment == "vast" and len(neutral_arguments) and neutral_count:
            instance = neutral_arguments.sample(1)
            neutral_count = neutral_count - 1
        elif len(con_arguments) and con_count:
            instance = con_arguments.sample(1)
            con_count = con_count - 1
        elif len(pro_arguments):
            instance = pro_arguments.sample(1)
            pro_count = pro_count - 1
        elif len(con_arguments):
            instance = con_arguments.sample(1)
            con_count = con_count - 1
        elif experiment == "vast" and len(neutral_arguments):
            instance = neutral_arguments.sample(1)
            neutral_count = neutral_count - 1
        else:
            break

        instance["cluster"] = cluster
        instances.append(instance)
    df_examples = pd.concat(instances)
    log_message(logger,f"found that count of pro arguments {len(df_examples[df_examples['stance'] == 1])}", level= logging.INFO)
    log_message(logger,f"found that count of con arguments {len(df_examples[df_examples['stance'] == 0])}", level= logging.INFO)
    log_message(logger,f"found that count of neutral arguments {len(df_examples[df_examples['stance'] == 2])}", level= logging.INFO)
    #df_examples = df_diverse_examples.groupby("cluster").sample(1)
    df_examples.sort_values("topic", inplace=True)
    return df_examples



import math
import random

import os
import json
import numpy as np
import pandas as pd

from scipy.spatial.distance import cosine
from collections import defaultdict

from tqdm import tqdm

# from prompting_stance import *
# from experiment import *
from few_shot_priming.argument_sampling.topic_similarity_sentence_transformer import *
from few_shot_priming.config import *
from few_shot_priming.utils import *
from few_shot_priming.experiments import *


os.environ["TOKENIZERS_PARALLELISM"] = "false"

def load_similarities(experiment, experiment_type, model="ctm"):
    path_similarities = get_similarities_path(experiment, experiment_type, model)
    with open(path_similarities, "r") as file:
        similarities= json.load(file)
    similarities_with_int_idices = {}
    for key in similarities:
        similarities_with_int_idices[int(key)]= {}
    for key in similarities:
        for train_key in similarities[key]:
            similarities_with_int_idices[int(key)][int(train_key)] = similarities[key][train_key]
    return similarities_with_int_idices


def save_similarities(experiment, experiment_type, similarities, model="ctm"):
    path_similarities = get_similarities_path(experiment, experiment_type, model)
    os.makedirs(os.path.dirname(path_similarities), exist_ok=True)

    with open(path_similarities, "w") as file:
        json.dump(similarities, file )
    print("succeed")

def evaluate_model(experiment, experiment_type, arguments_to_check, baseline=False):
    """
    :param model: contextual topic model to be evaluated

    """
    #similarities = load_similarities(experiment, experiment_type)
    if experiment_type =="validation":
        validate=True
    else:
        validate=False
    splits = load_splits(experiment, validate=validate, oversample=False)
    df_validation = splits[experiment_type]
    df_training = splits["training"]
    if baseline:
        sentence_transformer_similarities = load_similarities(experiment, experiment_type, model="sentence-transformer")
        #lda_similarities = calc_similarity_lda(df_validation, df_training)
    all_similar_examples = []
    arguments_to_check = df_validation.sample(arguments_to_check)
    for i, argument_record in arguments_to_check.iterrows():
        i = np.random.randint(0,len(df_validation))

        sentence_transformer_examples, sentence_transformer_score = sample_similar_examples(argument_record["id"], sentence_transformer_similarities, df_training, df_training.shape[0])
        sentence_transformer_examples.rename(columns={"text":"sentence-transformer-text", "topic":"sentence-transformer-topic"}, inplace=True)
        sentence_transformer_examples["sentence-transformer-score"] = sentence_transformer_score
        queries = [argument_record["text"] for _ in range(0,len(df_training))]
        queries_topic = [argument_record["topic"] for _ in range(0,len(df_training))]
        sentence_transformer_examples["query-text"] = queries
        sentence_transformer_examples["query-topic"] = queries_topic
        all_similar_examples.append(pd.concat([sentence_transformer_examples.reset_index()], axis=1))

    df_sorted_examples = pd.concat(all_similar_examples)
    df_sorted_examples.to_csv(f"~/contexutal_topic_model_{experiment}_{experiment_type}_evaluation.csv", sep="\t", columns=["query-text", "query-topic",
                                                                    #"ctm-text", "ctm-topic", "ctm-score",
                                                                    "sentence-transformer-text", "sentence-transformer-topic", "sentence-transformer-score"])




#memory = Memory("/tmp/similar-examples")
#@memory.cache
def sort_and_filter_hashmap(test_hashmap, df_training_indices):

    claims_with_similarity = sorted(test_hashmap.items(),key=lambda x: -x[1])
    items_indices = [claim_with_sim[0] for claim_with_sim in claims_with_similarity if claim_with_sim[0] in df_training_indices]
    return items_indices

def check_exists(instance, instances):
    for record in instances:
        if instance["arc_id"].values[0] == record["arc_id"].values[0]:
            return True
    return False


def sample_similar_examples(test_index, similarities, df_training, few_shot_size, experiment="ibmsc"):

    test_hashmap = similarities[test_index]
        #print(claims_with_similarity)
    df_training_indices = set(df_training["id"].values.tolist())

    items_indices = sort_and_filter_hashmap(test_hashmap, df_training_indices)

    df_pro_instances =  df_training[df_training["stance"]==1]
    df_con_instances =  df_training[df_training["stance"]==0]
    if experiment == "vast":
        df_neutral_instances = df_training[df_training["stance"]==2]
    if experiment == "ibmsc" or experiment == "perspectrum":
        class_few_shot_size = few_shot_size // 2
        pro_shot_size = class_few_shot_size
        con_shot_size = few_shot_size - pro_shot_size
    else:
        #con_shot_size, pro_shot_size, neutral_shot_size = decide_vast_shot_size(few_shot_size)
        con_shot_size, pro_shot_size, neutral_shot_size = few_shot_size/3, few_shot_size/3, few_shot_size/3
    if len(df_pro_instances)< pro_shot_size:
        df_pro_instances = df_pro_instances.sample(pro_shot_size, replace=True)
    if len(df_con_instances) < con_shot_size:
        df_con_instances = df_con_instances.sample(con_shot_size, replace=True)
    if experiment == "vast" and len(df_neutral_instances) < neutral_shot_size:
        df_neutral_instances = df_neutral_instances.sample(neutral_shot_size, replace=True)
    instances = []

    pro_instances_count = 0
    con_instances_count = 0
    neutral_instances_count = 0
    for index in items_indices:
        pro_instance = df_pro_instances[df_pro_instances["id"]==index]
        con_instance = df_con_instances[df_con_instances["id"]==index]
        if experiment == "vast":
            neutral_instances = df_neutral_instances[df_neutral_instances["id"]==index]
        if experiment == "vast" and len(neutral_instances) and neutral_instances_count < neutral_shot_size:
            if not check_exists(neutral_instances, instances):
                instances.append(neutral_instances)
                neutral_instances_count = neutral_instances_count + 1
        elif len(con_instance) and con_instances_count < con_shot_size:
            if (experiment =="ibmsc" or experiment == "perspectrum") or not check_exists(con_instance, instances):
                instances.append(con_instance)
                con_instances_count = con_instances_count +1
        elif len(pro_instance) and pro_instances_count<pro_shot_size:
            if (experiment =="ibmsc" or experiment == "perspectrum") or not check_exists(pro_instance, instances):
                instances.append(pro_instance)
                pro_instances_count = pro_instances_count + 1
        if (experiment =="ibmsc" or experiment == "perspectrum") and pro_instances_count + con_instances_count == few_shot_size:
            break
        if experiment == "vast" and pro_instances_count + con_instances_count + neutral_instances_count == few_shot_size:
            break
    df_few_shots = pd.concat(instances)
    df_few_shots["score"] = df_few_shots["id"].apply(lambda x: test_hashmap[x])
    df_few_shots.sort_values("topic", inplace=True)
    return df_few_shots, df_few_shots["score"]

def save_similar_examples_topic_counts(few_shot_size, experiment, save_text=False, debug=False):
    topic_counts = get_topic_count_range(experiment, False)

    similarities = load_similarities(experiment, "test", model="sentence-transformer")

    all_topic_count_instances = []
    for topic_count in tqdm(topic_counts):
        dataset = load_splits(experiment, validate= False, oversample=False,  topic_count= topic_count, debug=debug)
        for _, argument in tqdm(dataset["test"].iterrows()):
            test_id = argument["id"]
            similar_examples, scores = sample_similar_examples(test_id, similarities, dataset["training"], few_shot_size=few_shot_size, experiment=experiment)
            similar_examples["test-id"] = test_id
            similar_examples["topic-count"] = topic_count
            if save_text:
                similar_examples["test-text"] = argument["text"]
            all_topic_count_instances.append(similar_examples)
    path_examples_topic_counts = get_similar_examples_path(experiment, "test", topic_count=True)
    df_examples_topic_count = pd.concat(all_topic_count_instances)
    df_examples_topic_count.to_csv(path_examples_topic_counts, sep="\t", encoding="utf-8")

def save_similar_examples_ks( experiment, experiment_type, path_similar_examples, save_text=False, debug=False, drop_duplicate_perspectives=False):
    is_validate = experiment_type == "validation"
    similarities = load_similarities(experiment, experiment_type, model="sentence-transformer")
    print("loaded matrix")
    dataset = load_splits(experiment, oversample=False, validate=is_validate, debug=debug, drop_duplicate_perspectives=drop_duplicate_perspectives)
    print(f"loaded dataset with size {len(dataset)}")
    all_k_similar_instances = []
    for k in tqdm(get_ks(experiment)):
        for _, argument in tqdm(dataset[experiment_type].iterrows()):
            test_id = argument["id"]
            similar_examples, scores = sample_similar_examples(test_id, similarities, dataset["training"], few_shot_size=k, experiment=experiment)
            similar_examples["test-id"] = test_id
            similar_examples["k"] = k
            if save_text:
                similar_examples["test-text"] = argument["text"]
            all_k_similar_instances.append(similar_examples)

    df_examples_ks = pd.concat(all_k_similar_instances)
    df_examples_ks.to_csv(path_similar_examples, sep="\t", encoding="utf-8")


def load_similar_examples(experiment, experiment_type, path_similar_examples=None, topic_count=None, few_shot_size=16):
    if not path_similar_examples:
        path_similar_examples = get_similar_examples_path(experiment, experiment_type, topic_count=topic_count)
    df_examples = pd.read_csv(path_similar_examples, sep="\t")


    if topic_count:
        df_examples = df_examples[df_examples["topic-count"]==topic_count]
    elif few_shot_size:
        df_examples = df_examples[df_examples["k"]==few_shot_size]
    return df_examples



def get_similar_examples(test_index, df_examples):
    return df_examples[df_examples["test-id"]==test_index]


import json
import os
import pandas as pd
from few_shot_priming.config import *
from tqdm import tqdm
from few_shot_priming.experiments import *
from few_shot_priming.argument_sampling.stance_priming import *
# from config import *
import argparse


def adapt_ibmsc(df):
    labels = {"PRO": 1, "CON": 0}
    df["stance"] = df["claims.stance"].apply(lambda x: labels[x])
    df = df[["topicText", "claims.claimCorrectedText", "stance", "claims.claimId"]]
    df.rename(columns={"topicText": "topic", "claims.claimCorrectedText": "text", "claims.claimId": "id"}, inplace=True)
    return df[["topic", "stance", "text", "id"]]


def adapt_vast(df):
    df = df[["topic_str", "post", "label", "new_id", "arc_id"]]
    df.rename(columns={"topic_str": "topic", "post": "text", "label": "stance", "new_id": "id", "arc_id": "unique_id"},
              inplace=True)
    # df.drop_duplicates(subset=["text"], inplace=True)
    # df = df[df['stance'] != 2]
    return df[["topic", "stance", "text", "id", "unique_id"]]



def adapt_perspectrum(df):
    labels = {"SUPPORT": 1, "UNDERMINE": 0}
    df["stance"] = df["stance"].apply(lambda x: labels[x])
    return df[["topic", "stance", "text", "id"]]


def load_dataset(experiment):

    csv_file_train_path = f"../../experiment/{experiment}-train.csv"
    csv_file_val_path = f"../../experiment/{experiment}-validation.csv"
    csv_file_test_path = f"../../experiment/{experiment}-test.csv"

    df_training = pd.read_csv(csv_file_train_path)
    df_validation = pd.read_csv(csv_file_val_path)
    df_test = pd.read_csv(csv_file_test_path)

    return {"training": df_training, "validation": df_validation, "test": df_test}


def get_most_similar(ks, experiment):
    file_path = f"similarity/similarities-{experiment}-test-sentence-transformer.json"
    with open(file_path, "r") as file:
        similarities = json.load(file)

    dataset = load_dataset(experiment)
    for split in dataset:
        if experiment == "ibmsc":
            dataset[split] = adapt_ibmsc(dataset[split])
        elif experiment == "perspectrum":
            dataset[split] = adapt_perspectrum(dataset[split])
        else:
            dataset[split] = adapt_vast(dataset[split])
    csv_df = dataset["training"]
    val_id_list = list(dataset["validation"]["id"])
    train_id_list = list(dataset["training"]["id"])

    rows = []
    for key, similarity_dict in similarities.items():
        max_pids = sorted(similarity_dict, key=similarity_dict.get, reverse=True)
        max_pids = [id for id in max_pids if int(id) not in val_id_list]
        max_pids = [id for id in max_pids if int(id) in train_id_list]

        ks_count = 0
        for id in max_pids:
            topic = csv_df.loc[csv_df['id'] == int(id), 'topic'].iloc[0]
            text = csv_df.loc[csv_df['id'] == int(id), 'text'].iloc[0]
            stance = csv_df.loc[csv_df['id'] == int(id), 'stance'].iloc[0]

            row = {
                'topic': topic,
                'stance': stance,
                'text': text,
                'id': id,
                'score': similarity_dict[id],
                'test-id': key,
                'k': ks
            }
            rows.append(row)
            ks_count += 1
            if ks_count == ks:
                break

    df = pd.DataFrame(rows)
    save_file_path = f"sampling_strategies/{experiment}-most-similar.tsv"
    mode = 'a' if os.path.exists(save_file_path) else 'w'
    with open(save_file_path, mode, newline='') as file:
        df.to_csv(file, sep='\t', index=False, header=not os.path.exists(save_file_path))

def get_most_similar_balanced(ks, experiment):
    file_path = f"similarity/similarities-{experiment}-test-sentence-transformer.json"
    with open(file_path, "r") as file:
        similarities = json.load(file)

    dataset = load_dataset(experiment)
    split_n = 2
    for split in dataset:
        if experiment == "ibmsc":
            dataset[split] = adapt_ibmsc(dataset[split])
        elif experiment == "perspectrum":
            dataset[split] = adapt_perspectrum(dataset[split])
        else:
            dataset[split] = adapt_vast(dataset[split])
            split_n = 3

    csv_df = dataset["training"]
    val_id_list = list(dataset["validation"]["id"])
    train_id_list = list(dataset["training"]["id"])

    rows = []
    for key, similarity_dict in tqdm(similarities.items()):
        max_pids = sorted(similarity_dict, key=similarity_dict.get, reverse=True)
        max_pids = [id for id in max_pids if int(id) not in val_id_list]
        max_pids = [id for id in max_pids if int(id) in train_id_list]

        pro_count, con_count, neutral_count = 0, 0, 0
        pro_rows, con_rows, neutral_rows = [], [], []
        for id in max_pids:
            topic = csv_df.loc[csv_df['id'] == int(id), 'topic'].iloc[0]
            text = csv_df.loc[csv_df['id'] == int(id), 'text'].iloc[0]
            stance = csv_df.loc[csv_df['id'] == int(id), 'stance'].iloc[0]

            row = {
                'topic': topic,
                'stance': stance,
                'text': text,
                'id': id,
                'score': similarity_dict[id],
                'test-id': key,
                'k': ks
            }

            if stance == 1 and pro_count < ks / split_n:
                pro_rows.append(row)
                pro_count += 1
            elif stance == 0 and con_count < ks / split_n:
                con_rows.append(row)
                con_count += 1
            elif stance == 2 and neutral_count < ks / split_n:
                neutral_rows.append(row)
                neutral_count += 1

            if pro_count == ks / split_n and con_count == ks / split_n and (split_n == 2 or neutral_count == ks / split_n):
                rows.extend(pro_rows + con_rows + neutral_rows)
                break

    df = pd.DataFrame(rows)
    save_file_path = f"sampling_strategies/{experiment}-most-similar-balanced.tsv"
    mode = 'a' if os.path.exists(save_file_path) else 'w'
    with open(save_file_path, mode, newline='') as file:
        df.to_csv(file, sep='\t', index=False, header=not os.path.exists(save_file_path))


def get_most_similar_different_topics(experiment, drop_duplicates=False, max_threshold=None):
    config = load_config()
    file_path = config["topic-similarity"]["sentence-transformer-similarity-path"][f"{experiment}-test"]
    save_file_path = config["topic-similarity"]["similar-examples-ks"][f"{experiment}-test"]

    if drop_duplicates:
        save_file_path = save_file_path.replace(".tsv", "-no-duplicates.tsv")
    print(save_file_path)
    if experiment == "vast":
        ks = [3, 6, 12, 24, 48, 96]
    else:
        ks = [2, 4, 8, 16, 32, 64]

    #f"similarity/similarities-{experiment}-test-sentence-transformer.json"
    with open(file_path, "r") as file:
        similarities = json.load(file)
    ## drop duplicates apply only to perspectrum
    dataset =  load_splits(experiment, oversample=False, validate=False, drop_duplicate_perspectives=drop_duplicates)

    split_n = 2
    if experiment == "vast":
        split_n = 3
    csv_df = dataset["training"]
    #val_id_list = list(dataset["validation"]["id"])
    train_id_list = list(dataset["training"]["id"])

    rows = []
    for k in ks:
        for key, similarity_dict in tqdm(similarities.items()):
            max_pids = sorted(similarity_dict, key=similarity_dict.get, reverse=True)
            #max_pids = [id for id in max_pids if int(id) not in val_id_list]
            if max_threshold:
                max_pids = [id for id in max_pids if int(id) in train_id_list and similarity_dict[id]<=max_threshold]
            else:
                max_pids = [id for id in max_pids if int(id) in train_id_list]

            pro_rows, con_rows, neutral_rows = [], [], []
            pro_count, con_count, neutral_count = 0, 0, 0
            pro_topics, con_topics, neutral_topics = set(), set(), set()
            for id in max_pids:
                topic = csv_df.loc[csv_df['id'] == int(id), 'topic'].iloc[0]
                text = csv_df.loc[csv_df['id'] == int(id), 'text'].iloc[0]
                stance = csv_df.loc[csv_df['id'] == int(id), 'stance'].iloc[0]

                row = {
                    'topic': topic,
                    'stance': stance,
                    'text': text,
                    'id': id,
                    'score': similarity_dict[id],
                    'test-id': key,
                    'k': k
                }

                if stance == 1 and pro_count < k / split_n and topic not in pro_topics:
                    pro_rows.append(row)
                    pro_topics.add(topic)
                    pro_count += 1
                elif stance == 0 and con_count < k / split_n and topic not in con_topics:
                    con_rows.append(row)
                    con_topics.add(topic)
                    con_count += 1
                elif stance == 2 and neutral_count < k / split_n and topic not in neutral_topics:
                    neutral_rows.append(row)
                    neutral_topics.add(topic)
                    neutral_count += 1

                if pro_count == k / split_n and con_count == k / split_n and (split_n == 2 or neutral_count == k / split_n):
                    rows.extend(pro_rows + con_rows + neutral_rows)
                    break

    df = pd.DataFrame(rows)

    if max_threshold:
        save_file_path = save_file_path.replace(".tsv", f"-{max_threshold:.2}.tsv")
    if len(rows):
        df.to_csv(save_file_path, sep='\t', index=False)


def get_most_similar_majority_stance_different_topic_balanced(ks, experiment):
    file_path = f"similarity/similarities-{experiment}-test-sentence-transformer.json"
    with open(file_path, "r") as file:
        similarities = json.load(file)

    dataset = load_dataset(experiment)
    split_n = 2
    for split in dataset:
        if experiment == "ibmsc":
            dataset[split] = adapt_ibmsc(dataset[split])
        elif experiment == "perspectrum":
            dataset[split] = adapt_perspectrum(dataset[split])
        else:
            split_n = 3
            dataset[split] = adapt_vast(dataset[split])
    csv_df = dataset["training"]
    val_id_list = list(dataset["validation"]["id"])
    train_id_list = list(dataset["training"]["id"])

    rows = []
    for key, similarity_dict in similarities.items():
        max_pids = sorted(similarity_dict, key=similarity_dict.get, reverse=True)
        max_pids = [id for id in max_pids if int(id) not in val_id_list]
        max_pids = [id for id in max_pids if int(id) in train_id_list]
        max_100_pids = [id for id in max_pids if int(id) in train_id_list][:100]

        topic_count = {}
        for id in max_100_pids:
            topic = csv_df.loc[csv_df['id'] == int(id), 'topic'].iloc[0]
            if topic not in topic_count:
                topic_count[topic] = 0
            topic_count[topic] += 1
        top_topics = sorted(topic_count, key=topic_count.get, reverse=True)

        # If there are less than ks of top-topics, the loop recursively takes enough ks, top topics can repeat.
        top_topics_ks = []
        for count in range(max(ks, len(top_topics))):
            for topic in top_topics:
                if topic_count[topic] == 0:
                    continue
                topic_count[topic] -= 1
                top_topics_ks.append(topic)

        pro_rows, con_rows, neutral_rows = [], [], []
        temp_max_pids = []
        for topic in top_topics_ks:
            pro_rows_topic, con_rows_topic, neutral_rows_topic = [], [], []
            for id in max_pids:
                if csv_df.loc[csv_df['id'] == int(id), 'topic'].iloc[0] == topic:
                    temp_max_pids.append(id)
                    stance = csv_df.loc[csv_df['id'] == int(id), 'stance'].iloc[0]
                    text = csv_df.loc[csv_df['id'] == int(id), 'text'].iloc[0]
                    score = similarity_dict[id]
                    row = {
                        'topic': topic,
                        'stance': stance,
                        'text': text,
                        'id': id,
                        'score': score,
                        'test-id': key,
                        'k': ks
                    }
                    if stance == 1:
                        pro_rows_topic.append(row)
                    elif stance == 0:
                        con_rows_topic.append(row)
                    elif stance == 2:
                        neutral_rows_topic.append(row)
                    break
            max_pids = [item for item in max_pids if item not in temp_max_pids]

            pro_rows.extend(sorted(pro_rows_topic, key=lambda x: x['score'], reverse=True))
            con_rows.extend(sorted(con_rows_topic, key=lambda x: x['score'], reverse=True))
            neutral_rows.extend(sorted(neutral_rows_topic, key=lambda x: x['score'], reverse=True))

        rows.extend(pro_rows[:ks // split_n] + con_rows[:ks // split_n] + neutral_rows[:ks // split_n])

    df = pd.DataFrame(rows)
    save_file_path = f"sampling_strategies/{experiment}-most-similar-majority-stance-different-topic.tsv"
    mode = 'a' if os.path.exists(save_file_path) else 'w'
    with open(save_file_path, mode, newline='') as file:
        df.to_csv(file, sep='\t', index=False, header=not os.path.exists(save_file_path))


if __name__ == '__main__':
    # vast  perspectrum  ibmsc

    parser = argparse.ArgumentParser()
    parser.add_argument("--vast", action="store_true")
    parser.add_argument("--perspectrum", action="store_true")
    parser.add_argument("--ibmsc", action="store_true")
    parser.add_argument("--drop-duplicate-perspectives", action="store_true")
    parser.add_argument("--percentiles", type=int)
    args =  parser.parse_args()
    if args.vast:
        experiment = "vast"
    elif args.perspectrum:
        experiment = "perspectrum"
    elif args.ibmsc:
        experiment = "ibmsc"
    else:
        experiment = None

    if experiment:
        if args.percentiles:
            config = load_config()
            file_path = config["topic-similarity"]["sentence-transformer-similarity-path"][f"{experiment}-test"]
            thresholds = get_thresholds_for_percentiles(file_path, args.percentiles)
            for threshold in thresholds:
                get_most_similar_different_topics(experiment=experiment, drop_duplicates=args.drop_duplicate_perspectives, max_threshold=threshold)
        else:
            get_most_similar_different_topics(experiment=experiment, drop_duplicates=args.drop_duplicate_perspectives)

    else:
        for possible_e in ["perspectrum", "ibmsc", "vast"]:
            get_most_similar_different_topics(experiment=possible_e, drop_duplicates=args.drop_duplicate_perspectives)
            print(f"finished-{possible_e}")


    # get_most_similar(ks=24, experiment="vast")
    # get_most_similar_balanced(ks=24, experiment="vast")
    # get_most_similar_different_topics(ks=24, experiment="vast")
    # get_most_similar_majority_stance_different_topic(ks=24, experiment="vast")

    # get_most_similar_majority_stance_different_topic_balanced(16, "perspectrum")
    # get_most_similar_majority_stance_different_topic_balanced(16, "vast")
    # get_most_similar_majority_stance_different_topic_balanced(16, "ibmsc")
    # get_most_similar(ks=2, experiment="perspectrum")
    # get_most_similar(ks=4, experiment="perspectrum")
    # get_most_similar(ks=8, experiment="perspectrum")



import os
import setuptools
from setuptools import find_packages



new_version = "0.3.938"

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "few_shot_priming",
    version = new_version,
    author = "Yamen Ajjour",
    packages = ["few_shot_priming", "few_shot_priming.argument_sampling"],
    author_email = "yajjour@hotmail.com",
    description = "Analyzing priming effects in a few shot setting environment",
    package_data={'': ['experiment/*.*' , 'logs/tmp', 'data/*.*', "conf.yaml", "conf-google.yaml"]},
    package_dir={ 'few_shot_priming': '.'},
    long_description = long_description,
    include_package_data=True,
    long_description_content_type = "text/markdown",
    url = "https://gitlab.uni-hannover.de/y.ajjour/few-shot-priming",
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
print (new_version)



import pandas as pd
import json
from collections import defaultdict
from tqdm import tqdm
from few_shot_priming.experiments import load_splits
from sentence_transformers import SentenceTransformer, losses, SentencesDataset, datasets, util
from sentence_transformers.evaluation import BinaryClassificationEvaluator
from sentence_transformers.readers import InputExample
from scipy.spatial.distance import cosine

def preprocess_dataset(dataset):
    for split in dataset:
        dataset[split]["topic-text"] = dataset[split].apply(lambda record: record["topic"] + "[SEP]" + record["text"], axis=1)

def generate_test_dataset(df, three_classes=False):
    list1 = []
    list2 = []
    labels = []
    for topic, df_topic in df.groupby("topic"):
        df_stances = list(df_topic.groupby("stance"))
        if len(df_stances)<2 or (three_classes and len(df_stances)<3)   :
            continue
        df_stance_1 = df_stances[0][1]
        df_stance_2 = df_stances[1][1]
        if not len(df_stance_1) or not len(df_stance_2):
            continue

        if three_classes:
            df_stance_3 = df_stances[2][1]
            if not len(df_stance_3) and not len(df_stance_2):
                continue

        for i, argument in df_topic.iterrows():
            for j, argument_2 in df_topic.iterrows():
                if j>i:
                    list1.append(argument["topic-text"])
                    list2.append(argument_2["topic-text"])
                    if argument["stance"] == argument_2["stance"]:
                        label = 1
                    else:
                        label = 0
                    labels.append(label)

    return list1, list2, labels

def train(model, params, train_dataloader, evaluator=None, output_model_path=None):


    learning_rate = params["lr"]
    epochs = params["epochs"]
    train_loss = losses.ContrastiveLoss(model=model)
    model.fit(
        train_objectives=[(train_dataloader, train_loss)],
        optimizer_params = {'lr' : learning_rate},
        output_path=output_model_path,
        epochs=epochs,
        evaluator=evaluator,
        save_best_model=True,
        evaluation_steps=1000
    )
    return model

def create_evaluator(df_test, three_examples=False):
    list1, list2, labels = generate_test_dataset(df_test, three_examples)
    evaluator = BinaryClassificationEvaluator(list1, list2, labels,show_progress_bar=True)
    return evaluator


def generate_training_examples(df_training, three_classes=False):
    contrastive_examples = []
    guid = 0
    for topic, df_topic in df_training.groupby("topic"):
        df_stances = list(df_topic.groupby("stance"))
        if len(df_stances)<2 or (three_classes and len(df_stances)<3):
            continue
        df_stance_1 = df_stances[0][1]
        df_stance_2 = df_stances[1][1]

        if not len(df_stance_1) or not len(df_stance_2):
            continue
        if three_classes:
            df_stance_3 = df_stances[2][1]
            if not len(df_stance_3) or not len(df_stance_2):
                continue
        for i, argument in df_topic.iterrows():
            for j, argument_2 in df_topic.iterrows():
                if j>i:
                    pair = [argument["topic-text"], argument_2["topic-text"]]
                    if argument["stance"] == argument_2["stance"]:
                        label = 1
                    else:
                        label = 0
                    guid = guid + 1
            contrastive_examples.append(InputExample(texts=pair, label=label, guid=guid))

    return contrastive_examples

def load_similarities(path_similarities):

    with open(path_similarities, "r") as file:
        similarities= json.load(file)
    similarities_with_int_idices = {}
    for key in similarities:
        similarities_with_int_idices[int(key)]= {}
    for key in similarities:
        for train_key in similarities[key]:
            similarities_with_int_idices[int(key)][int(train_key)] = similarities[key][train_key]
    return similarities_with_int_idices

def generate_similarity_matrix(experiment, path_model, path_similarities, debug=False):

    model = SentenceTransformer(path_model)
    dataset =  load_splits(experiment, oversample=False, validate=False )
    preprocess_dataset(dataset)
    test_text = dataset["test"]["topic-text"]
    training_text = dataset["training"]["topic-text"]
    test_embeddings = model.encode(test_text.values.tolist())
    training_embeddings = model.encode(training_text.values.tolist())
    cosine_scores = util.cos_sim(test_embeddings, training_embeddings)
    similarities = defaultdict(dict)
    i,j = 0, 0
    for _,test_record in tqdm(dataset["test"].iterrows()):
        for _,train_record in dataset["training"].iterrows():
            similarities[test_record["id"]][train_record["id"]] = float(cosine_scores[i,j])
            j = j + 1
        i = i + 1
        j = 0
    with open(path_similarities, "w") as file:
        json.dump(similarities, file )


def generate_most_similar_stance_arguments(experiment, path_similarities, path_samples, different_topics, debug, max_threshold=None, drop_duplicate_perspectives=False):

    similarities = load_similarities(path_similarities)
    dataset =  load_splits(experiment, oversample=False, validate=False, drop_duplicate_perspectives=drop_duplicate_perspectives, debug=debug)
    df_training = dataset["training"]

    all_samples = []
    if experiment == "vast":
        ks = [3, 6, 12, 24, 48, 96]
    else:
        ks = [2, 4, 8, 16, 32, 64]

    if different_topics:
        for _, test_record in tqdm(dataset["test"].iterrows()):
            instances_scores_map = similarities[test_record["id"]]
            elements_to_drop = []
            if  max_threshold:
                for key in instances_scores_map:
                    if instances_scores_map[key] >max_threshold:
                        elements_to_drop.append(key)
                for key in elements_to_drop:
                    del instances_scores_map[key]

            df_training_subset = df_training[df_training["id"].isin(instances_scores_map.keys())]
            df_training_subset["score"] = df_training_subset["id"].apply(lambda id: instances_scores_map[id])
            training_instances = list(df_training_subset.itertuples(index=False))
            training_instances.sort(key=lambda x: x.score, reverse=True)

            for k in ks:
                topic_set = set()
                id_set = set()
                samples = []
                exists_enough_topics = True
                if len(training_instances) < k:
                    k = len(training_instances)
                #if len(training_instances) == 0:
                    #raise ValueError
                while len(id_set) < k:


                    for training_record in training_instances:
                        if exists_enough_topics and training_record.topic not in topic_set and training_record.id not in id_set:
                            id_set.add(training_record.id)
                            topic_set.add(training_record.topic)
                        elif not exists_enough_topics and training_record.id not in id_set:
                            id_set.add(training_record.id)
                            topic_set.add(training_record.topic)
                        if len(id_set) == k:
                            samples = []
                            for instance in training_instances:
                                if instance.id in id_set:
                                    samples.append(instance)
#                                    exists = False
#                                    for sample in samples:
#                                        if instance.id == sample.id:
#                                            exists=True
#                                            break
#                                    if not exists:

                            break
                    if len(id_set) < k:
                        exists_enough_topics = False

                df_samples = pd.DataFrame(samples)
                df_samples["k"] = k
                df_samples["test-id"] = test_record["id"]
                all_samples.append(df_samples)

    else:
        for _, test_record in dataset["test"].iterrows():
            instances_scores_map = similarities[test_record["id"]]
            if  max_threshold:
                elements_to_drop = []
                for key in instances_scores_map:
                    if instances_scores_map[key] >=max_threshold:
                        elements_to_drop.append(key)
                for key in elements_to_drop:
                        del instances_scores_map[key]
            instances_scores = list(zip(instances_scores_map.keys(), instances_scores_map.values()))
            instances_scores.sort(key = lambda x: x[1], reverse=True)
            for k in ks:
                top_indices, top_scores = zip(*instances_scores[:k])
                samples = df_training[df_training["id"].isin(top_indices)]
                samples.drop_duplicates(["id"], inplace=True) # in perspectrum dataset some entries are duplicate
                #print(test_record["id"])
                samples["test-id"] = test_record["id"]
                samples["score"]=top_scores
                samples["k"]=k
                all_samples.append(samples)
    df_samples = pd.concat(all_samples)
    if max_threshold:
        path_samples = path_samples.replace(".tsv", f"-{max_threshold:.2}.tsv")
    df_samples.to_csv(path_samples, sep="\t", encoding="utf-8", columns=["topic", "stance", "text", "id", "score", "test-id", "k"])

def get_thresholds_for_percentiles(path_similarities, percentile_num):
    similarities = load_similarities(path_similarities)
    similarity_list = []
    for test_index in similarities:
        similarity_list.extend(similarities[test_index].values())
    percentile_size = len(similarity_list) // percentile_num
    indices = range(0,len(similarity_list),percentile_size)
    print(indices)
    similarity_list = sorted(similarity_list)
    thrsholds = [similarity_list[i] for i in list(indices)]

    return thrsholds

def get_pathes_for_thresholds(path_sample_root, path_similarities, percentile_num):
    thresholds = get_thresholds_for_percentiles(path_similarities, percentile_num)
    threshold_pathes_list = []
    for threshold in thresholds:
        path_samples = path_sample_root.replace(".tsv", f"-{threshold:.2}.tsv")
        threshold_pathes_list.append((threshold, path_samples))
    return threshold_pathes_list




import pandas as pd
import os
import math
from few_shot_priming.config import *
from few_shot_priming.argument_sampling.topic_similarity import *


def save_splits():
    """
    Save the splits of the experiments by sampling a validation set from the training set with exclusive topic sets
    :return:
    """
    path_source, path_training, path_validation, path_test = get_experiment_paths()
    df_arguments = pd.read_csv(path_source, sep=",", encoding="utf-8")
    df_training = df_arguments[df_arguments["split"] == "train"]
    df_test = df_arguments[df_arguments["split"] == "test"]
    training_topics = df_training["topicTarget"].sample(12).unique()
    training_topics = training_topics[:10]
    df_sampled_training = df_training[df_training["topicTarget"].isin(training_topics)]
    df_validation = df_training[~df_training["topicTarget"].isin(training_topics)]
    df_sampled_training.to_csv(path_training, sep=",", encoding="utf-8", index=False)
    df_validation.to_csv(path_validation, sep=",", encoding="utf-8", index=False)
    df_test.to_csv(path_test, sep=",", encoding="utf-8", index=False)
    #save_splits_topic_counts()
def save_splits_topic_counts():

    for experiment in ["ibmsc", "vast"]:
        path_source, path_training, path_validation, path_test = get_experiment_paths(experiment)
        df_training = pd.read_csv(path_training, sep=",", encoding="utf-8")
        df_validation = pd.read_csv(path_validation, sep=",", encoding="utf-8")
        df_training["split"] = "training"
        df_validation["split"] = "validation"
        df_all_training = pd.concat([df_training, df_validation])

        if experiment == "ibmsc":
            topic_label = "topicText"
        else:
            topic_label = "topic_str"
        is_validate = False
        topic_count_range = get_topic_count_range(experiment, is_validate)

        for topic_count in topic_count_range:
            df_all_training[f"topic-count-{topic_count}"]=False
            topics = df_all_training[topic_label].unique()
            chosen_topics = np.random.choice(topics, topic_count, replace=False)
            df_all_training[f"topic-count-{topic_count}"] = df_all_training[topic_label].apply(lambda x: x in chosen_topics)

        df_training = df_all_training[df_all_training["split"] == "training"]
        df_validation = df_all_training[df_all_training["split"] == "validation"]
        del df_training["split"]
        del df_validation["split"]
        df_training.to_csv(f"/home/yamen/projects/few-shot-priming/experiment/{experiment}-train.csv", sep=",", encoding="utf-8", index=False)
        df_validation.to_csv(f"/home/yamen/projects/few-shot-priming/experiment/{experiment}-validation.csv", sep=",", encoding="utf-8", index=False)

def oversample_dataset(df, experiment):

    if experiment == "ibmsc" or experiment == "perspectrum":
        if experiment == "ibmsc":
            stance_label = "claims.stance"
            pro_label = "Pro"
            con_label = "Con"
        else:
            stance_label = "stance"
            pro_label = "SUPPORT"
            con_label = "UNDERMINE"
        pro_claims = df[df[stance_label]==pro_label]
        con_claims = df[df[stance_label]==con_label]
        count_of_con_to_sample = pro_claims.shape[0] - con_claims.shape[0]
        if count_of_con_to_sample > 0:
            if count_of_con_to_sample <= len(con_claims):
                con_claims_to_add = con_claims.sample(count_of_con_to_sample)
            else:
                con_claims_to_add = con_claims.sample(count_of_con_to_sample, replace=True)
            df = pd.concat([df, con_claims_to_add])
        elif count_of_con_to_sample < 0:
            count_of_pro_to_sample = - count_of_con_to_sample
            if count_of_pro_to_sample <= len(pro_claims):
                pro_claims_to_add = pro_claims.sample(count_of_pro_to_sample)
            else:
                pro_claims_to_add = pro_claims.sample(count_of_pro_to_sample, replace=True)
            df = pd.concat([df, pro_claims_to_add])
        else:
            pass
        return df
    else:
        max_label_count = df["label"].value_counts().max()
        all_labels_to_add = []
        for label, df_label in df.groupby("label"):
            instances_to_sample = max_label_count -  len(df_label)
            if instances_to_sample <= len(df_label):
                df_label_to_add = df_label.sample(instances_to_sample)
            else:
                df_label_to_add = df_label.sample(instances_to_sample, replace=True)
            all_labels_to_add.append(df_label_to_add)
        all_labels_to_add.append(df)
        return pd.concat(all_labels_to_add)

def adapt_ibmsc(df):
    labels = {"PRO": 1, "CON":0}
    df["stance"] = df["claims.stance"].apply(lambda x: labels[x])
    df = df [["topicText", "claims.claimCorrectedText", "stance", "claims.claimId"]]
    df.rename(columns={"topicText" : "topic", "claims.claimCorrectedText": "text", "claims.claimId": "id"}, inplace=True)
    return df[["topic", "stance", "text", "id"]]

def adapt_vast(df):
    df = df[["topic_str", "post", "label", "new_id", "arc_id"]]
    df.rename(columns={"topic_str":"topic", "post": "text", "label": "stance", "new_id":"id", "ori_id": "unique_id"}, inplace=True)
    #df["topic"] = df["topic"].apply(lambda topic: " ".join(topic))
    #df["stance"] = df["stance"].apply(lambda stance: stance.caplitalize())
    return df[["topic", "stance", "text", "id", "arc_id"]]

def adapt_perspectrum(df):
    labels = {"SUPPORT": 1, "UNDERMINE":0}
    df["stance"] = df["stance"].apply(lambda x: labels[x])
    return df

def load_splits(experiment, oversample=True, validate=True, drop_duplicate_perspectives=False, adapt=True, topic_count=None, debug=False):
    """
    Load the splits of the experiments and return it in a dictionary of pandas dataframes
    :return: a dictionary containing the training, validation, and test splits
    """
    if experiment == "ibmsc":
        dataset_id="claims.claimId"
        topic_label = "topicText"
    elif experiment == "perspectrum":
        dataset_id="id"
        topic_label = "topic"
    else:
        dataset_id = "new_id"
        topic_label = "topic_str"

    path_source, path_training, path_validation, path_test = get_experiment_paths(experiment)
    if experiment == "ibmsc" and not os.path.exists(path_validation):
        save_splits()
    df_training = pd.read_csv(path_training, sep=",", encoding="utf-8")
    df_validation = pd.read_csv(path_validation, sep=",", encoding="utf-8")
    df_test = pd.read_csv(path_test, sep=",", encoding="utf-8")
    if experiment =="vast":
        if validate:
            df_validation = df_validation[df_validation["seen?"]==0]
        df_test = df_test[df_test["seen?"]==0]

    if experiment == "perspectrum":

        df_training = pd.read_csv(path_training, sep=",", encoding="utf-8")
        df_validation = pd.read_csv(path_validation, sep=",", encoding="utf-8")
        df_test = pd.read_csv(path_test, sep=",", encoding="utf-8")
        if drop_duplicate_perspectives:
            df_training = df_training[~df_training["perspective_id"].isin(df_test["perspective_id"])]
            df_validation = df_validation[~df_validation["perspective_id"].isin(df_test["perspective_id"])]
    if debug:
        #df_training = df_training.sample(400)
        df_validation = df_validation.sample(frac=0.1)
        df_test = df_test.sample(frac=0.1)

    if not validate:
        df_training = pd.concat([df_training, df_validation])
        if topic_count:
            df_training = df_training[df_training[f"topic-count-{topic_count}"]]

    dataset = {"training": df_training, "validation": df_validation, "test": df_test}

    if oversample:
        dataset["training"] = oversample_dataset(dataset["training"], experiment)
    else:
        pass
    for split in dataset:
        dataset[split].sort_values(by=dataset_id, inplace=True)
    if adapt:
        for split in dataset:
            if experiment == "ibmsc":
                dataset[split] = adapt_ibmsc(dataset[split])
            elif experiment == "perspectrum":
                dataset[split] = adapt_perspectrum(dataset[split])
            else:
                dataset[split] = adapt_vast(dataset[split])

    return dataset


def split_train_val_by_topic(part_size, experiment, split):
    train_csv_path = f"../experiment/{experiment}-train.csv"
    val_csv_path = f"../experiment/{experiment}-validation.csv"

    if experiment == "ibmsc":
        train_df = adapt_ibmsc(pd.read_csv(train_csv_path))
        val_df = adapt_ibmsc(pd.read_csv(val_csv_path))
    elif experiment == "perspectrum":
        train_df = adapt_perspectrum(pd.read_csv(train_csv_path))
        val_df = adapt_perspectrum(pd.read_csv(val_csv_path))
    elif experiment == "vast":
        train_df = adapt_vast(pd.read_csv(train_csv_path))
        val_df = adapt_vast(pd.read_csv(val_csv_path))

    if split == "merge":
        df = pd.concat([train_df, val_df], ignore_index=True)
    elif split == "train":
        df = train_df
    elif split == "val":
        df = val_df

    unique_claims = df['topic'].unique()
    num_parts = math.ceil(len(unique_claims) / part_size)

    for i in range(1, num_parts):
        df[f'topic-{i * part_size}'] = False

    for i in range(num_parts):
        start_idx = 0
        end_idx = (i + 1) * part_size
        if i == num_parts - 1:
            df['topic-all'] = True
            break
        claims_in_part = unique_claims[start_idx:end_idx]
        for claim in claims_in_part:
            df.loc[df['topic'] == claim, f'topic-{(i + 1) * part_size}'] = True

    output_csv_path = f"../experiment/split/{experiment}-split-{split}-by-topic.csv"
    df.to_csv(output_csv_path, index=False)

def save_experiment_results(experiment, df_run_results, run_name):
    path_experiment_results = get_results_path(experiment)
    df_run_results["run"] = run_name
    now = datetime.now()
    time_now = now.strftime("%m-%d-%H:%M")
    df_run_results["time"] = time_now


    if os.path.exists(path_experiment_results):
        df_results = pd.read_csv(path_experiment_results, sep="\t", encoding="utf-8")
        df_new_results = pd.concat([df_run_results, df_results])
    else:
        df_new_results = df_run_results
    df_new_results.to_csv(path_experiment_results, sep= "\t", encoding="utf-8", index=False)






from datetime import datetime
import os
import yaml

from pathlib import Path


root_path = Path(__file__).parent

def get_absolute_path(path):
    return Path(root_path, path)

def adjust_config(config):
    for key in config:
        if isinstance(config[key],str):
            config[key] = config[key].replace("/bigwork/nhwpajjy","/mnt/home/yajjour")
        elif isinstance(config[key],dict):
            adjust_config(config[key])
        else:
            pass
def adjust_config_to_kisski():
    config = load_config()
    adjust_config(config)
    save_config(config)

def load_config():
    """
    Load the configuration of the experiment and the model
    :return: a dictionary containing the configuration of the experiments
    """

    conf_path = Path(root_path, "conf.yaml")
    with open(conf_path) as file:
        config = yaml.safe_load(file)
        return config

def save_config(config):
    conf_path = Path(root_path, "conf.yaml")
    with open(conf_path, "w") as file:
        yaml.dump(config, file)

def get_prompting_config():
    config = load_config()
    return config["prompt"]


def get_prompt_fine_tuning():
    """
    Load the prompting approach main configuration
    :return:
    """
    config = load_config()
    return config["prompt-fine-tuning"]

def get_prompt_fine_tuning_params(model):
    """
    Load the prompt params to optimize  the model
    :return: a dictionary containing the params for the few shot model
    """
    prompt_fine_tuning_config = get_prompt_fine_tuning()
    return prompt_fine_tuning_config[f"{model}-params"]

def get_prompt_fine_tuning_best_params(experiment, model):
    """
    Load the prompting approach best params
    :return:
    """
    prompting_config = get_prompt_fine_tuning()
    return prompting_config[f"{model}-best-params"][experiment]

def get_contrastive_embeddings_params(experiment):
    config = load_config()
    return config["stance-priming"]["best-params"][experiment]

def get_experiment_paths(experiment):
    conf = load_config()
    if experiment == "ibmsc":
        path_source = Path(root_path, conf["dataset"]["path-ibmsc-root"])
    elif experiment == "perspectrum":
        path_source = None
    else:
        path_source = None
    path_training = Path(root_path, conf["experiment"][experiment]["path-training"])
    path_test = Path(root_path, conf["experiment"][experiment]["path-test"])
    if experiment in ["ibmsc", "perspectrum", "vast", "snopes"]:
        path_validation = Path(root_path, conf["experiment"][experiment]["path-validation"])
    else:
        path_validation = None
    return path_source, path_training, path_validation, path_test


def get_baseline_params():
    config = load_config()
    return config["baseline"]["params"]

def get_baseline_config():
    config = load_config()
    config = config["baseline"]
    return config

def get_baseline_best_params(experiment):
    config = load_config()
    return config["baseline"]["best-params"][experiment]

def get_logs(experiment, run):
    config = load_config()
    home_directory = os.path.expanduser( '~' )
    now = datetime.now()
    time_now = now.strftime("%m-%d-%H:%M")
    path = config["experiment"][experiment]["path-logs"].replace("time", time_now).replace("run", run)
    return Path(home_directory, path)

def get_run_results_path(experiment=None, run=""):
    config = load_config()
    home_directory = os.path.expanduser( '~' )
    now = datetime.now()
    time_now = now.strftime("%m-%d-%H:%M")
    path = config["experiment"][experiment]["path-run-results"].replace("time", time_now)
    path = path.replace("run", run)
    return Path(home_directory, path)

def get_results_path(experiment=None, run=""):
    config = load_config()
    home_directory = os.path.expanduser( '~' )
    now = datetime.now()
    time_now = now.strftime("%m-%d-%H:%M")
    path_results = config["experiment"][experiment]["path-results"].replace("time", time_now)
    return Path(home_directory, path_results)

def get_openai_key():
    config = load_config()
    return config["openai"]["key"]

def load_topic_similarity_params():
    config = load_config()
    return config["topic-similarity"]["params"]

def get_topic_model_path(experiment, experiment_type):
    config = load_config()
    token = f"{experiment}-{experiment_type}"
    return config["topic-similarity"]["model-path"][token]

def get_similarities_path(experiment, experiment_type, model):

    config = load_config()
    token = f"{experiment}-{experiment_type}"
    return config["topic-similarity"][f"{model}-similarity-path"][token]

def dump_bow_size(experiment, experiment_type, bow_size):
    config = load_config()
    token = f"{experiment}-{experiment_type}"
    config["topic-similarity"]["bow"][token] = bow_size
    save_config(config)

def load_bow_size(experiment, experiment_type):
    config = load_config()
    token = f"{experiment}-{experiment_type}"
    return config["topic-similarity"]["bow"][token]

def get_template_path():
    conf = load_config()
    return Path(root_path, conf["prompt-fine-tuning"]["path-template"])

def get_diverse_example_path(experiment, experiment_type, topic_count=None, drop_duplicates=False):
    config = load_config()
    token = f"{experiment}-{experiment_type}"
    if drop_duplicates and experiment != "perspectrum":
        raise ValueError("duplicates are only considered for the perspectrum dataset")

    if topic_count:
        return config["topic-similarity"]["diverse-examples-topics"][token]
    elif drop_duplicates:
        return config["topic-similarity"]["diverse-examples"][f"{token}-no-duplicates"]
    else:
        return config["topic-similarity"]["diverse-examples"][token]

def get_similar_examples_path(experiment, experiment_type, topic_count=None):
    config = load_config()
    token = f"{experiment}-{experiment_type}"
    if topic_count:
         return  config["topic-similarity"]["similar-examples-topics"][token]
    else:
        return  config["topic-similarity"]["similar-examples-ks"][token]

def get_gpt2_params():
    return {
    "model-input-limit": 1024,
    "model-name": "gpt2",
    "model-path": "gpt2",
    "model-type": "gpt2"
    }

def get_seeds():
    conf = load_config()
    return conf["seeds"]

def get_ks(experiment):
    conf = load_config()
    return conf["experiment"][experiment]["ks"]

def get_few_shot_size(experiment):
    conf = load_config()
    return conf["experiment"][experiment]["few-shot-size"]

def get_analyze_k_results_pathes():
    conf = load_config()
    return conf["analyze-k"]

def get_analyze_topic_count_results_pathes():
    conf = load_config()
    return conf["analyze-topic-count"]

def get_analyze_topic_count_k():
    conf = load_config()
    return conf["analyze-topic-count"]["k"]

def get_topic_similarity_logs_path():
    conf = load_config()
    return conf["topic-similarity"] ["path-logs"]

def get_count_of_samples_per_cluster():
    conf = load_config()
    return conf["topic-similarity"]["count-of-samples-per-cluster"]


